import copy
from html.parser import HTMLParser
from typing import List, Union

from . import ContentTextToken, TextStyle, StyleTextToken
from .TextToken import TextToken


class TextTokenizer(HTMLParser):

    def __init__(self, style: TextStyle):
        super(TextTokenizer, self).__init__()
        self.__currentStyle = copy.deepcopy(style)
        self.__previousStyles = []  # type: List[TextStyle]
        self.__tokens = []  # type: List[TextToken]
        self.__tokens.append(StyleTextToken(self.__currentStyle))

    def parse(self, content: str) -> List[TextToken]:
        self.feed(content)
        return self.__tokens

    def handle_starttag(self, tag, attrs):
        if tag == "s":
            self.__previousStyles.append(self.__currentStyle)
            self.__currentStyle = copy.deepcopy(self.__currentStyle)
            for attr in attrs:
                if attr[0] == "color":
                    self.__currentStyle.color = hex_to_rgb(attr[1])
                elif attr[0] == "size":
                    tileHeight = int(attr[1])
                    self.__currentStyle.tileHeight = tileHeight
                    self.__currentStyle.updateShifts(tileHeight)
                else:
                    raise RuntimeError("Invalid or unsupported attribute {} in tag {}".format(attr, tag))
        else:
            self.__currentStyle = copy.deepcopy(self.__currentStyle)
            if tag == "i":
                self.__currentStyle.italic = True
            elif tag == "b":
                self.__currentStyle.bold = True
            elif tag == "u":
                self.__currentStyle.underline = True
            elif tag == "shadow":
                self.__currentStyle.shadow = True
                for attr in attrs:
                    if attr[0] == "color":
                        self.__currentStyle.shadowColor = hex_to_rgb(attr[1])
                    else:
                        raise RuntimeError("Invalid or unsupported attribute {} in tag {}".format(attr, tag))
            elif tag == "outline":
                self.__currentStyle.outline = True
                for attr in attrs:
                    if attr[0] == "color":
                        self.__currentStyle.outlineColor = hex_to_rgb(attr[1])
                    else:
                        raise RuntimeError("Invalid or unsupported attribute {} in tag {}".format(attr, tag))
            else:
                raise RuntimeError("Invalid or unsupported tag {}".format(tag))
        self.__tokens.append(StyleTextToken(self.__currentStyle))

    def handle_endtag(self, tag):
        if tag == "s":
            self.__currentStyle = self.__previousStyles.pop()
        else:
            self.__currentStyle = copy.deepcopy(self.__currentStyle)
            if tag == "i":
                self.__currentStyle.italic = False
            elif tag == "b":
                self.__currentStyle.bold = False
            elif tag == "u":
                self.__currentStyle.underline = False
            elif tag == "shadow":
                self.__currentStyle.shadow = False
            elif tag == "outline":
                self.__currentStyle.outline = False
            else:
                raise RuntimeError("Invalid or unsupported tag {}".format(tag))
        self.__tokens.append(StyleTextToken(self.__currentStyle))

    def handle_data(self, content):
        self.__tokens.append(ContentTextToken(content))


def hex_to_rgb(value):
    value = value.lstrip('#')
    lv = len(value)
    return tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))
